<?php

namespace EventPortal\Domain\Component;

abstract class Input extends Component
{
    /** @var null|string|array */
    public $value;

    /** @var null|string */
    public $placeholder;

    /** @var bool */
    public $clearable = false;

    /** @var bool */
    public $required = false;

    /** @var array */
    public $validation;

    /** @var array */
    protected $validationRules;

    public function hasOptions(): bool
    {
        return false;
    }

    /**
     * @param string|array $value
     * @return $this
     */
    public function value($value): self
    {
        $this->value = $value;

        return $this;
    }

    public function validation(?array $rules): self
    {
        if ($rules === null) {
            return $this;
        }

        $this->validation = $rules;

        if (in_array('required', $rules, true)) {
            $this->required = true;
        }

        return $this;
    }

    public function placeholder(?string $placeholder): self
    {
        $this->placeholder = $placeholder;

        return $this;
    }

    public function clearable(?bool $clearable): self
    {
        $this->clearable = $clearable ?? false;

        return $this;
    }

    /**
     * Returns the component as an array.
     */
    public function toArray(): array
    {
        $data = array_merge(parent::toArray(), [
            'clearable' => $this->clearable,
            'required' => $this->required,
        ]);

        if ($this->placeholder !== null) {
            $data['placeholder'] = $this->placeholder;
        }

        if ($this->value !== null) {
            $data['value'] = $this->value;
        }

        if ($this->validationRules !== null) {
            $data['validation'] = $this->validationRules;
        }

        if ($data['value'] === null) {
            unset($data['value']);
        }

        return $data;
    }
}
