<?php

namespace EventPortal\Http\Controllers\Admin;

use EventPortal\Domain\Component\Button;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Form\Facades\Form;
use EventPortal\Domain\Table\Facades\Table;
use EventPortal\Enums\InertiaMethod;
use EventPortal\Http\Controllers\Controller;
use Illuminate\Http\RedirectResponse;
use Inertia\Inertia;
use Inertia\Response;

class EventController extends Controller
{
    public function index(): Response
    {
        $table = Table::make(Event::class)
            ->columns(request('cols'))
            ->size(request('size'))
            ->page(request('page'))
            ->search(request('query'))
            ->filters(request('filter'))
            ->get();

        return Inertia::render('Event/Index', [
            'data' => $table,
        ]);
    }

    public function create(): Response
    {
        $form = Form::make('event')
            ->addAction(function () {
                return Button::make('Cancel')
                    ->variant('secondary')
                    ->method(InertiaMethod::VISIT())
                    ->route('admin.event.index');
            })
            ->addAction(function () {
                return Button::make('Create Event')
                    ->method(InertiaMethod::POST())
                    ->route('admin.event.store');
            });

        return Inertia::render('Event/Create', [
            'form' => $form,
        ]);
    }

    public function store(): RedirectResponse
    {
        Form::make('event')
            ->withRequest(request())
            ->validate()
            ->submit();

        return redirect()->route('admin.event.index');
    }

    public function edit(Event $event): Response
    {
        $form = Form::make($event)
            ->addAction(function () {
                return Button::make('Cancel')
                    ->variant('secondary')
                    ->method(InertiaMethod::VISIT())
                    ->route('admin.event.index');
            })
            ->addAction(function () use ($event) {
                return Button::make('Update Event')
                    ->method(InertiaMethod::POST())
                    ->route('admin.event.update', ['event' => $event]);
            });

        return Inertia::render('Event/Create', [
            'form' => $form,
        ]);
    }

    public function update(Event $event): RedirectResponse
    {
        Form::make($event)
            ->withRequest(request())
            ->validate()
            ->update();

        return redirect()->route('admin.event.index');
    }
}
